//
// "$Id: fl_file_dir.cxx,v 1.1.2.2 2001/09/30 17:37:06 easysw Exp $"
//
// File chooser widget for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2001 by Bill Spitzak and others.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "fltk-bugs@fltk.org".
//

#include <config.h>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_File_Chooser.H>

static Fl_File_Chooser	*fc = (Fl_File_Chooser *)0;
static void		(*current_callback)(const char*) = 0;


static void callback(Fl_File_Chooser *, void*) {
  if (current_callback)
    (*current_callback)(fc->value(0));
}


void fl_file_chooser_callback(void (*cb)(const char*)) {
  current_callback = cb;
}


char* fl_file_chooser(const char* message, const char* pat, const char* fname)
{
  if (!fname || !*fname) fname = ".";

  if (!fc) {
    fc = new Fl_File_Chooser(fname, pat, Fl_File_Chooser::CREATE, message);
    fc->callback(callback, 0);
  } else {
    fc->type(Fl_File_Chooser::CREATE);
    fc->filter(pat);
    fc->value(fname);
    fc->label(message);
  }

  fc->show();

  while (fc->visible())
    Fl::wait();

  return ((char *)fc->value());
}


char* fl_dir_chooser(const char* message, const char* fname)
{
  if (!fname || !*fname) fname = ".";

  if (!fc) {
    fc = new Fl_File_Chooser(fname, "*", Fl_File_Chooser::CREATE |
                                         Fl_File_Chooser::DIRECTORY, message);
    fc->callback(callback, 0);
  } else {
    fc->type(Fl_File_Chooser::CREATE | Fl_File_Chooser::DIRECTORY);
    fc->filter("*");
    fc->value(fname);
    fc->label(message);
  }

  fc->show();

  while (fc->visible())
    Fl::wait();

  return ((char *)fc->value());
}


//
// End of "$Id: fl_file_dir.cxx,v 1.1.2.2 2001/09/30 17:37:06 easysw Exp $".
//
